
;+

pro trace_align_cube, itrace, dtrace, index, data, $
                      ss_ref, ss_start, ss_end,  $
                      missing = missing, nodiffrot=nodiffrot, $
                      center=center, dimensions=dimensions,  $
                      scale=scale, date=date, $
                      verbose=verbose, quiet=quiet

;NAME:
;   TRACE_ALIGN_CUBE
;PURPOSE:
;   Dejitter/align a TRACE data cube and optionally correct for solar
;   rotation. 
;CATEGORY:
;CALLING SEQUENCE:
;   trace_align_cube, itrace, dtrace, index, data, ss_ref, ss_start, ss_end
;INPUTS:
;   itrace, dtrace = original TRACE index and data cube.  You must
;                    correct the pointing (.xcen,.ycen) in itrace
;                    before running trace_align_cube if you want to
;                    remove jitter (see trace_mdi_align and
;                    trace_cube_pointing).
;OPTIONAL INPUT PARAMETERS:
;   ss_ref = index of the reference image in itrace,dtrace.  All other
;            images are aligned to this one.  Default = 0.
;   ss_start, ss_end = If you like, you can operate on a subset of the
;                      data cube.  These give the index of the start
;                      and end images in itrace,dtrace.  Dafault =
;                      align the full data cube.  
;KEYWORD PARAMETERS:
;   missing = set value of extrapolated pixels to this number.
;   /nodiffrot = do not apply solar rotation to the shifts.
;   center, dimensions, scale, date = Rather than aligning to a reference
;                              image, align all images to center =
;                              [x,y], dimension = [xsize,ysize], and
;                              scale = [yscale,yscale] at date = date.
;OUTPUTS:
;   index, data = the new TRACE index and data cube.
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     The .xcen and .ycen fields of the index are assumed to be
;     correct.  You should use trace_mdi_align and trace_cube_pointing
;     to make sure this is true before using trace_align_cube.  If you
;     don't do this, the jitter will *not* be removed.  This might still
;     be useful, depending on your application, but beware.
;PROCEDURE:
;     Determines the image shifts from the pointing in the TRACE
;     header (.xcen and .ycen) and then uses shift_img to do the
;     alignments.
;
;     If the images in the data cube are not all the same size or
;     scale, the final data cube will have the size and scale of the
;     reference image. 
;MODIFICATION HISTORY:
;     T. Metcalf  2003-Sep-24
;-

if n_elements(ss_start) LE 0 then $
   ss_start  = 0                     ; Start index in original data cube
if n_elements(ss_end) LE 0 then $
   ss_end = n_elements(itrace)-1     ; Final index in original data cube
if n_elements(ss_ref) LE 0 then $
   ss_ref = 0                        ; Reference index in original data cube
if n_elements(itrace) NE n_elements(dtrace[0,0,*]) then $
   message,'ERROR: the number in index records does not equal the number of images'

if n_elements(center) GE 2 then begin
   xcenter = center[0]
   ycenter = center[1]
   if keyword_set(verbose) then message,/info,'Using user supplied center'
endif else begin
   if n_elements(center) GE 1 then $
      message,/info,'One element center passed in is being ignored'
   xcenter = itrace[ss_ref].xcen
   ycenter = itrace[ss_ref].ycen
   if keyword_set(verbose) then message,/info,'Using reference image center'
endelse
if n_elements(scale) GE 2 then begin
   xscale = scale[0]
   yscale = scale[1]
   if keyword_set(verbose) then message,/info,'Using user supplied scale'
endif else begin
   if n_elements(scale) GE 1 then $
      message,/info,'One element scale passed in is being ignored'
   xscale = itrace[ss_ref].cdelt1
   yscale = itrace[ss_ref].cdelt2
   if keyword_set(verbose) then message,/info,'Using reference image scale'
endelse
if n_elements(dimensions) GE 2 then begin
   xsize = dimensions[0]
   ysize = dimensions[1]
   if keyword_set(verbose) then message,/info,'Using user supplied dimensions'
endif else begin
   if n_elements(dimensions) GE 1 then $
      message,/info,'One element dimensions passed in is being ignored'
   xsize = itrace[ss_ref].naxis1
   ysize = itrace[ss_ref].naxis2
   if keyword_set(verbose) then $
      message,/info,'Using reference image dimensions'
endelse
if n_elements(date) GE 1 then begin
   date = anytim(date) 
   if keyword_set(verbose) then message,/info,'Using user supplied date'
endif else begin
   date = anytim(itrace[ss_ref].date_obs)
   if keyword_set(verbose) then message,/info,'Using reference image date'
endelse
lonlat = xy2lonlat([xcenter,ycenter],date)

nd = ss_end - ss_start + 1
index = itrace[ss_start:ss_end]
data = fltarr(xsize,ysize,nd)

; Shift the images to the reference image

if NOT keyword_set(quiet) then $
   progress,0.0,/reset,label='trace_align_cube:'
for i=ss_start,ss_end do begin
   if NOT keyword_set(nodiffrot) then begin
      ; compute the shift due to solar rotation
      dateold = anytim(itrace[i].date_obs)
      ddays = (date - dateold)/86400.
      dlon= diff_rot(ddays,lonlat[1],/allen)
      lonlatold = xy2lonlat([itrace[i].xcen,itrace[i].ycen],dateold)
      lonlatold[0] = lonlatold[0] + dlon
      xyold = lonlat2xy(lonlatold,dateold)
      ; diffshift is the x-y shift (pixels) due to solar rotation
      diffshift = (xyold - [itrace[i].xcen,itrace[i].ycen]) / $
                  [itrace[i].cdelt1,itrace[i].cdelt2]
   endif else diffshift = [0.0,0.0]
   nxold = itrace[i].naxis1
   nyold = itrace[i].naxis2
   xscalefac = xscale/itrace[i].cdelt1
   yscalefac = yscale/itrace[i].cdelt2
   ; The shift in pixels due to shift in the center of the image,
   ; the size of the image, and solar rotation.
   ; Here I assume that the trace pointing has been corrected
   xshift = (itrace[i].xcen - xcenter)/itrace[i].cdelt1 + $
            ((xsize-1)*xscalefac - (nxold-1))/2.0 + $
            diffshift[0]
   yshift = (itrace[i].ycen - ycenter)/itrace[i].cdelt2 + $
            ((ysize-1)*yscalefac - (nyold-1))/2.0 + $
            diffshift[1]
   imold = fltarr(xsize>nxold,ysize>nyold)
   imold[0,0] = float(dtrace[0:nxold-1,0:nyold-1,i])
   imnew = shift_img(imold,[xshift,yshift], $
                     xscale=itrace[i].cdelt1/xscale, $
                     yscale=itrace[i].cdelt2/yscale, $
                     missing=missing)
   data[*,*,i-ss_start] = imnew[0:xsize-1,0:ysize-1]
   index[i-ss_start].xcen = xcenter - diffshift[0]
   index[i-ss_start].ycen = ycenter - diffshift[1]
   index[i-ss_start].crval1 = index[i-ss_start].xcen
   index[i-ss_start].crval1 = index[i-ss_start].ycen 
   index[i-ss_start].crpix1 = (xsize+1)/2.0
   index[i-ss_start].crpix1 = (ysize+1)/2.0
   index[i-ss_start].naxis1 = xsize
   index[i-ss_start].naxis2 = ysize
   index[i-ss_start].cdelt1 = xscale
   index[i-ss_start].cdelt2 = yscale
   if NOT keyword_set(quiet) then $
      progress,100.0*float(i-ss_start+1)/float(ss_end-ss_start+1),freq=5.
endfor
if NOT keyword_set(quiet) then $
   progress,100.0,/last
update_history,index,'aligned to image from '+anytim(date,/yohkoh)

; Done with alignment: data and index now hold the new, 
; aligned data cube and index

end
